package MakeWorld::Utils;

use strict;
use warnings;

use File::Spec::Functions qw[ catfile file_name_is_absolute ];

use base 'Exporter';

use Module::Pluggable require => 1;
use Module::Load qw[ load ];
use MakeWorld::Global;

our @EXPORT_OK =
  qw[
	find_exe
	find_plugins
   ];


sub find_exe
{
    my ( $exe ) = @_;

    my $file;

    # check if absolute path to file; test directly
    if ( file_name_is_absolute( $exe ))
    {
	$file = $exe if  -f $exe && -x _;
    }

    # not absolute; find it
    else
    {
	( $file ) = grep { -f $_ && -x _}
	  map { glob(catfile($_,$exe)) }
	    @{$Opt->srcdir}, split(':', $ENV{PATH} ), '.';
    }

    die( "unable to find executable command: $exe \n" )
	unless $file;

    return $file;
}



sub find_plugins {

    my ( $pkg, $plugins, $map, @dist ) = @_;

    __PACKAGE__->search_path( new => $pkg );

    @{$plugins} = __PACKAGE__->plugins();
    %{$map}     = map { my $module = $_;
		    $module =~ s/^.*:://;
		    ( lc($module) => $_ ) } @{$plugins};

    # Module::Pluggable won't find fatpacked plugins, so explicitly
    # load the distributed ones
    for my $plugin ( @dist ) {

	my $fq_plugin = $pkg . '::' . $plugin;
	$plugin = lc $plugin;
	next if exists $map->{$plugin};
	load $fq_plugin;
	push @{$plugins}, $fq_plugin;
	$map->{$plugin} = $fq_plugin;
    }

    return;
}


1;
